#!/usr/bin/perl -w
#
# AVR DTMF tone generator table creator, (w) Axel XL Schwenke
#
# usage:
# make_tables.pl [XTAL [SCALE [N_SAMPLES [PRESCALER]]]] > dtmf_tables.h
#
# $Id: make_tables.pl 217 2013-10-17 18:23:57Z schwenke $

use strict;

#defaults
my $XTAL= 8000000;  #AVR clock frequency
my $PRESCALER=1;    #timer 1 prescaler
my $SCALE=3;        #scaling used for CurSinValA/B
my $N_SAMPLES= 128; #samples in sinus table
#end defaults

if (@ARGV) {
    $XTAL = shift @ARGV;
}

if (@ARGV) {
    $SCALE = shift @ARGV;
}

if (@ARGV) {
    $N_SAMPLES = shift @ARGV;
}

if (@ARGV) {
    $PRESCALER = shift @ARGV;
}

my @DTMF_H= (1209, 1336, 1477, 1633);
my @DTMF_L= (697, 770, 852, 941);

print "/* DTMF tables generated by $0\n";
print " * Parameters in use:\n";
print " * XTAL      = $XTAL\n";
print " * PRESCALER = $PRESCALER\n";
print " * SCALE     = $SCALE\n";
print " * N_SAMPLES = $N_SAMPLES\n";
print " */\n\n";

print "#define N_SAMPLES ", fit_in_uchar($N_SAMPLES), "\n";
print "#define SCALE     $SCALE\n";
print "#define SHIFT     ", int(2**($SCALE-1)), "\n";

print "\nconst unsigned char PROGMEM sin_table [$N_SAMPLES] =\n{\n";
for (my $i = 0; $i<$N_SAMPLES; $i++) {
    if ($i) {
        print ",", ($i % 16 == 0 ? "\n" : " ");
    }
    print fit_in_uchar(63.5 + 64 * sin(2*3.1416*$i/$N_SAMPLES));
}
print "\n};\n\n";

print "const unsigned char dtmf_freqL [4] = { ";
for (my $i = 0; $i<4; $i++) {
    print ", " if ($i);
    print fit_in_uchar(0.5 + 510.0 * $DTMF_L[$i] * $N_SAMPLES * 2**$SCALE * $PRESCALER / $XTAL);
}
print " };\n\n";

print "const unsigned char dtmf_freqH [4] = { ";
for (my $i = 0; $i<4; $i++) {
    print ", " if ($i);
    print fit_in_uchar(0.5 + 510.0 * $DTMF_H[$i] * $N_SAMPLES * 2**$SCALE * $PRESCALER / $XTAL);
}
print " };\n\n";


sub fit_in_uchar
{
    my $val = shift;
    $val = int($val);

    if ($val < 0 or $val > 255) {
        warn "value $val does not fit in 'unsigned char'\n";
    }
    return $val;
}
